Imports Microsoft.VisualBasic
Imports System.ComponentModel

Public Class ComplexConverter

   'Dieser Separator kann Fehler verursachen, wenn Text gespeichert wird, der '|' enthlt
   'Wird hier nur fr die Demo verwendet
   Private Const Separator As Char = "|"c

   'Dieser Separator ist sicher, aber nicht darstellbar in der Richtextbox
   'Private Const Separator As Char = ChrW(&HF019)

    ''' <summary>
    ''' zeigt an, da beim Restaurieren ein behandelbarer Fehler aufgetreten ist
    ''' </summary>
    Public Class RestoreException : Inherits Exception
        Public Sub New(ByVal ex As Exception)
            MyBase.New("Restore failed", ex)
        End Sub
    End Class

    Public Class EventArg : Inherits EventArgs

        ' Das Event ComplexConverter.Convert(), welches dieses EventArg verwendet, wird in zwei
        ' Modi ausgefhrt:
        ' Store konvertiert einen gegebenen Wert und fgt der IList(Of String) einen Eintrag hinzu
        ' Restore re-konvertiert einen Eintrag und berschreibt damit den gegebenen Wert

        Private _VBProvider As VBCodeProvider
        Private _Strings As IList(Of String)
        Private _Counter As Integer = 0
        Public ReadOnly IsStoring As Boolean

        Sub New(ByVal Provider As VBCodeProvider, ByVal Strings As IList(Of String), _
                ByVal IsStoring As Boolean)
            _VBProvider = Provider
            _Strings = Strings
            Me.IsStoring = IsStoring
        End Sub

        ''' <summary>Hier angegebene Werte werden gespeichert und restauriert</summary>
        <Description("Hier angegebene Werte werden gespeichert und restauriert")> _
        Public Sub ConvertValue(Of T)(ByRef Item As T)
            If IsStoring Then
                Dim S As String = _VBProvider.GetConverter(GetType(T)).ConvertToString(Item)
                If S.Contains(Separator) Then
                    Dim Msg As String = String.Concat( _
                        "Der zu speichernde String '", S, _
                        "' darf nicht den ComplexConverter.Separator-Char (", _
                        System.Convert.ToString(AscW(Separator), 16), _
                        ") enthalten.")
                    Throw New ArgumentException(Msg, "Item")
                End If
                _Strings.Add(S)
            Else
                Item = GetValue(Of T)()
            End If
        End Sub

        ''' <summary>gibt einen gespeicherten Wert zurck</summary>
        <Description("gibt einen gespeicherten Wert zurck")> _
        Public Function GetValue(Of T)() As T
            If IsStoring Then
                Throw New Exception( _
                    ".GetValue() darf nicht aufgerufen werden, wenn .IsStoring = True.")
            Else
                Try
                    Dim C As TypeConverter = _VBProvider.GetConverter(GetType(T))
                    GetValue = DirectCast(C.ConvertFromString(_Strings(_Counter)), T)
                Catch ex As Exception
                    Throw New RestoreException(ex)
                End Try
                _Counter += 1
            End If
        End Function

        ' Sonderfall ConvertList(): Bei Listen wird die Anzahl Count der Elemente gespeichert.
        ' Restore leert die Liste und fgt dann Count uninitialisierte Elemente hinzu.

        ''' <summary>speichert Anzahl, restauriert mit uninitialisierten Elementen</summary>
        <Description("speichert Anzahl, restauriert mit uninitialisierten Listenelementen")> _
        Public Sub ConvertList(Of TItem As New)(ByVal List As IList)
            If IsStoring Then
                _Strings.Add(List.Count.ToString)
            Else
                List.Clear()
                Try
                    For I As Integer = 0 To Integer.Parse(_Strings(_Counter)) - 1
                        List.Add(New TItem)
                    Next
                Catch ex As Exception
                    Throw New RestoreException(ex)
                End Try
                _Counter += 1
            End If
        End Sub

        Public Sub ConvertConverter(ByVal Converter As ComplexConverter)
            ' ComplexConverters listigste Zeile: ermglicht, den Konvertierungs-Vorgang an einen
            ' anderen Converter abzugeben, dabei aber "meine" _Strings zu verwenden. So kann ein
            ' UserControl aus einer anderen Assembly seine eigene Konvertierung definieren, die
            ' dann in den Settings der Entry-Assembly mit-persistiert wird
            Converter.OnConvert(Me)
        End Sub

    End Class ' ComplexConverter.EventArg

    Public Event Convert As EventHandler(Of EventArg)

    Private Sub OnConvert(ByVal e As EventArg)
        RaiseEvent Convert(Me, e)
    End Sub

    Public Function CreateDataString() As String
        Dim Strings As New List(Of String)
        Using VBProvider As New VBCodeProvider
            ' Store: Die im Ereignishandler angegebenen Items mit vom VBProvider erhaltenen
            ' TypeConvertern nach Strings konvertieren
            RaiseEvent Convert(Me, New EventArg(VBProvider, Strings, True))
        End Using
        Return String.Join(Separator, Strings.ToArray)
    End Function

    Public Sub ApplyDataString(ByVal S As String)
        If S = "" Then Return
        Dim Backup As New List(Of String)
        Using VBProvider As New VBCodeProvider
            'Ist-Zustand ins Backup speichern
            OnConvert(New EventArg(VBProvider, Backup, IsStoring:=True))
            Try
                'Restaurierung anhand von S versuchen
                OnConvert(New EventArg(VBProvider, S.Split(Separator), False))
            Catch Ex As RestoreException
                'Im Fehlerfall Re-Restaurierung anhand des Backups
                OnConvert(New EventArg(VBProvider, Backup, False))
            End Try
        End Using
    End Sub


End Class

